var mapsProvider = (function() {
	return new function() {
		
		this.ESRI_STREET = 'ESRI Street';
		this.ESRI_AERIAL = 'ESRI Aerial';
		this.MAPQUEST_STREET = 'MapQuest Street';
		this.MAPQUEST_AERIAL = 'MapQuest Aerial';
		this.OPENSTREETMAP_STREET = 'OpenStreetMap Street'; 
		this.YAHOO_STREET = 'Yahoo Street';
		this.YAHOO_AERIAL = 'Yahoo Aerial';
		this.YAHOO_HYBRID = 'Yahoo Hybrid';
		this.GOOGLE_STREET = 'Google Maps';
		this.BING_AERIAL = 'Bing Aerial';
		this.BING_HYBRID = 'Bing Hybrid';
		this.BING_STREET = 'Bing Street';

		var providers = {};

		providers[this.ESRI_STREET] = {
			name: this.ESRI_STREET,
			layer: L.tileLayer('http://server.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer/tile/{z}/{y}/{x}')
		};

		providers[this.ESRI_AERIAL] = {
			name: this.ESRI_AERIAL,
			layer: L.tileLayer('http://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}')
		};

		providers[this.MAPQUEST_STREET] = {
			name: this.MAPQUEST_STREET,
			layer: L.tileLayer('http://otile1.mqcdn.com/tiles/1.0.0/map/{z}/{x}/{y}.jpg')
		};

		providers[this.MAPQUEST_AERIAL] = {
			name: this.MAPQUEST_AERIAL,
			layer: L.tileLayer('http://otile1.mqcdn.com/tiles/1.0.0/sat/{z}/{x}/{y}.jpg')
		};

		providers[this.OPENSTREETMAP_STREET] = {
			name: this.OPENSTREETMAP_STREET,
			layer: L.tileLayer('http://tile.openstreetmap.org/{z}/{x}/{y}.png')
		};

		providers[this.GOOGLE_STREET] = {
			name: this.GOOGLE_STREET,
			layer: new L.Google('ROADMAP')
		};

		providers[this.BING_AERIAL] = {
			name: this.BING_AERIAL,
			layer: new BingLayer('http://a{s}.ortho.tiles.virtualearth.net/tiles/a{q}.jpeg?g=90', {
				subdomains: ['0', '1', '2', '3']
			})
		};

		providers[this.BING_HYBRID] = {
			name: this.BING_HYBRID,
			layer: new BingLayer('http://h{s}.ortho.tiles.virtualearth.net/tiles/h{q}.jpeg?g=90', {
				subdomains: ['0', '1', '2', '3']
			})
		};

		providers[this.BING_STREET] = {
			name: this.BING_STREET,
			layer: new BingLayer('http://r{s}.ortho.tiles.virtualearth.net/tiles/r{q}.png?g=90&shading=hill', {
				subdomains: ['0', '1', '2', '3']
			})
		};

		providers[this.YAHOO_STREET] = {
			name: this.YAHOO_STREET,
			layer: L.tileLayer('{url}', {
				url: function(params)
				{
					var y = params.y,
						x = params.x,
						z = 18 - params.z;

					var temp = Math.pow(2.0, params.z) / 2.0;

					if (y < temp)
					{
						y = (temp - y) - 1.0;
					}
					else
					{
						y = ((y + 1) - temp) * -1.0;
					}

					var url = 'http://us.maps2.yimg.com/us.png.maps.yimg.com/png?v=3.52&t=m&' +
							  'x=' + x + '&' +
							  'y=' + y + '&' +
							  'z=' + z;
					
					return url;
				}
			})
		};

		providers[this.YAHOO_AERIAL] = {
			name: this.YAHOO_AERIAL,
			layer: L.tileLayer('{url}', {
				url: function(params)
				{
					var y = params.y,
						x = params.x,
						z = 18 - params.z;

					var temp = Math.pow(2.0, params.z) / 2.0;

					if (y < temp)
					{
						y = (temp - y) - 1.0;
					}
					else
					{
						y = ((y + 1) - temp) * -1.0;
					}

					var url = 'http://us.maps3.yimg.com/aerial.maps.yimg.com/tile?v=1.7&t=a&' +
							  'x=' + x + '&' +
							  'y=' + y + '&' +
							  'z=' + z;

					return url;
				}
			})
		};

		providers[this.YAHOO_HYBRID] = {
			name: this.YAHOO_HYBRID,
			layer: L.layerGroup([
				L.tileLayer('{url}', {
					url: function(params)
					{
						var y = params.y,
							x = params.x,
							z = 18 - params.z;

						var temp = Math.pow(2.0, params.z) / 2.0;

						if (y < temp)
						{
							y = (temp - y) - 1.0;
						}
						else
						{
							y = ((y + 1) - temp) * -1.0;
						}

						var url = 'http://us.maps3.yimg.com/aerial.maps.yimg.com/tile?v=1.7&t=a&' +
								  'x=' + x + '&' +
								  'y=' + y + '&' +
								  'z=' + z;

						return url;
					}
				}), 
				L.tileLayer('{url}', {
					url: function(params)
					{
						var y = params.y,
							x = params.x,
							z = 18 - params.z;

						var temp = Math.pow(2.0, params.z) / 2.0;

						if (y < temp)
						{
							y = (temp - y) - 1.0;
						}
						else
						{
							y = ((y + 1) - temp) * -1.0;
						}

						var url = 'http://us.maps3.yimg.com/aerial.maps.yimg.com/png?v=2.2&t=h&' +
								  'x=' + x + '&' +
								  'y=' + y + '&' +
								  'z=' + z;

						return url;
					}
				})
			])
		};

		this.getProvider = function(provider)
		{
			if (providers[provider] !== undefined)
			{
				return providers[provider];
			}
		}

		this.getProviders = function()
		{
			return providers;
		}

		this.setProviderSelected = function(provider)
		{
			if (providers[provider] === undefined)
			{
				throw new Error('El proveedor de mapas ' + provider + ' no existe');
			}

			if (typeof(Storage) !== 'undefined')
			{
			    localStorage.setItem('LastProviderSelected', provider);
			}
		}

		this.getLastProviderSelected = function()
		{
			var provider = undefined;

			if (typeof(Storage) !== 'undefined')
			{
				var data = localStorage.getItem('LastProviderSelected');
				
				if (data !== undefined)
				{
					provider = providers[data];
				}
			}

			return provider;
		}

		this.delProviderSelected = function()
		{
			localStorage.removeItem('LastProviderSelected');
		}
	}
})();